
#ifndef __LITMUS_SEM_H__
#define __LITMUS_SEM_H__

/* a semaphore maps to a single lso, as with queue locks... if you want to use
 * these as group locks, use an set of resources as  the lso and give the
 * semaphore an appropriate name (e.g., resources A through D in same lso,
 * name lock "ABCD").
 */

/* semaphore handles should not be modified by users */
typedef void*  lso_sem_t;

/* lso_create_sem(handle, name) - create and initialize new semaphore
 *
 * This function is guaranteed to behave atomically, i.e., to processes racing
 * to create the same semaphore will not intefere in  unpredictable ways. The
 * slower process will always fail in trying to create an already existing semaphore.
 */
lso_error_t lso_create_sem(lso_t handle, const char* name);

/* lso_get_sem(handle, name, sem_handle) - obtain a handle for a semaphore
 *
 * A handle is needed for locking and unlocking operations. Returns LSO_OK on
 * success. Stores handle in *sem_handle.
 */
lso_error_t lso_get_sem(lso_t handle, const char* name, lso_sem_t* sem_handle);

/* lso_semlock(handle) - enter critical section
 */
void lso_semlock(lso_sem_t handle);

/* lso_semunlock(handle) - exit critical section
 */
void lso_semunlock(lso_sem_t handle);

/* lso_release_sem(handle) - release the _handle_ to a sema and sem itself
 */
void lso_release_sem(lso_sem_t *handle);

#endif
