
#ifndef __SO_SRP_SEM_H__
#define __SO_SRP_SEM_H__

#include "so_internal_arch.h"
#include "litmus_so.h"
#include "so_qlock.h"
#include "litmus.h"

/*
 * Most of the functions that follow are "wrappers" for system calls. This is the way
 * we can use the shared object library *and* do most of the semaphore implementation
 * inside of the kernel.
 */

static inline void __srp_sem_lock_init(srp_sema_id *sem)
{
	*sem = srp_sema_init();
}

static inline void __srp_semaphore_lock(struct litmus_so* lso, srp_sema_id *sem)
{
 	srp_down(*sem);
}

static inline void __srp_semaphore_unlock(struct litmus_so* lso, srp_sema_id *sem)
{	
	srp_up(*sem);
}

static inline void __srp_semaphore_reg(struct litmus_so* lso, srp_sema_id *sem)
{
	reg_task_srp_sem(*sem, getpid());
}

static inline void __srp_semaphore_reg_other(struct litmus_so* lso, srp_sema_id *sem,
		                             pid_t task_pid)
{
	reg_task_srp_sem(*sem, task_pid);
}

lso_global_t __lso_malloc(struct litmus_so* lso, unsigned int size);
void __lso_free(struct litmus_so* lso, lso_global_t ptr);

static inline lso_global_t __alloc_srp_semaphore(struct litmus_so* lso) 
{
	lso_global_t semlock;
	srp_sema_id* _semlock;

	semlock  = __lso_malloc(lso, sizeof(srp_sema_id));

	if (!semlock)
		return 0;
	
	_semlock    = lso_local_ptr((lso_t) lso, semlock);
	__srp_sem_lock_init(_semlock);

	if (*_semlock < 0)
		return 0;

	return semlock;
}

static inline void __free_srp_semaphore(struct litmus_so* lso, lso_global_t semlock)
{
	srp_sema_id* sem;
	sem = lso_local_ptr((lso_t) lso, semlock);
	srp_sema_free(*sem);
	__lso_free(lso, semlock);	
}

static inline lso_global_t alloc_srp_semaphore(struct litmus_so* lso) 
{
	lso_global_t ret;
	__lso_lock(lso);
	ret = __alloc_srp_semaphore(lso);
	__lso_unlock(lso);
	return ret;
}

static inline void free_srp_semaphore(struct litmus_so* lso, lso_global_t semlock) 
{
	__lso_lock(lso);
	__free_srp_semaphore(lso, semlock);
	__lso_unlock(lso);
}

#endif
