#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "litmus_so.h"
#include "litmus_pi_sem.h"
#include "so_pi_sem.h"

struct pi_sem {
	struct litmus_so* 	lso;
	pi_sema_id*		lock; /* ID of appropriate pi_semaphore in litmus kernel */
};

lso_error_t lso_create_pi_sem(lso_t handle, const char* name)
{
	struct litmus_so* lso = (struct litmus_so*) handle;
	lso_global_t semlock;
	char buf[32];	

	snprintf(buf, 32, "pi-sem/%s", name);
	
	/* check if it exists already */
	if (lso_get(handle, buf))
		return LSO_EXISTS;
	semlock = alloc_pi_semaphore(lso);
	if (lso_put_exclusive(handle, buf, semlock) != LSO_OK) {
		free_pi_semaphore(lso, semlock);
		return LSO_EXISTS;
	} else
		return LSO_OK;
}


lso_error_t lso_get_pi_sem(lso_t handle, const char* name, lso_pi_sem_t* sem_handle)
{
	struct pi_sem* sem;
	lso_global_t semlock;
	char buf[32];	

	snprintf(buf, 32, "pi-sem/%s", name);
	semlock = lso_get(handle, buf);

	if (!semlock)
		return LSO_NOT_FOUND;
	sem = (struct pi_sem*) malloc(sizeof(struct pi_sem));
	if (!sem)
		return LSO_NO_MEM;
	sem->lso  = (struct litmus_so*) handle;
	sem->lock = lso_local_ptr(handle, semlock);
	*sem_handle = sem;
	return LSO_OK;
}

void lso_pi_semlock(lso_pi_sem_t handle)
{
	struct pi_sem* sem = (struct pi_sem*) handle;
	__pi_semaphore_lock(sem->lso, sem->lock);
}


void lso_pi_semunlock(lso_pi_sem_t handle)
{
	struct pi_sem* sem = (struct pi_sem*) handle;
	__pi_semaphore_unlock(sem->lso, sem->lock);
}


void lso_release_pi_sem(lso_pi_sem_t *handle)
{
	struct pi_sem* sem = (struct pi_sem*) *handle;
	pi_sema_free(*(sem->lock));
	free(sem);
	*handle = NULL;
}

