#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "litmus_so.h"
#include "litmus_sem.h"
#include "so_sem.h"

struct sema {
	struct litmus_so* 	lso;
	sema_id*		lock; /* ID of appropriate semaphore in litmus kernel */
};

lso_error_t lso_create_sem(lso_t handle, const char* name)
{
	struct litmus_so* lso = (struct litmus_so*) handle;
	lso_global_t semlock;
	char buf[32];	

	snprintf(buf, 32, "sem/%s", name);
	
	/* check if it exists already */
	if (lso_get(handle, buf))
		return LSO_EXISTS;
	semlock = alloc_semaphore(lso);
	if (lso_put_exclusive(handle, buf, semlock) != LSO_OK) {
		free_semaphore(lso, semlock);
		return LSO_EXISTS;
	} else
		return LSO_OK;
}


lso_error_t lso_get_sem(lso_t handle, const char* name, lso_sem_t* sem_handle)
{
	struct sema* sem;
	lso_global_t semlock;
	char buf[32];	

	snprintf(buf, 32, "sem/%s", name);
	semlock = lso_get(handle, buf);

	if (!semlock)
		return LSO_NOT_FOUND;
	sem = (struct sema*) malloc(sizeof(struct sema));
	if (!sem)
		return LSO_NO_MEM;
	sem->lso  = (struct litmus_so*) handle;
	sem->lock = lso_local_ptr(handle, semlock);
	*sem_handle = sem;
	return LSO_OK;
}

void lso_semlock(lso_sem_t handle)
{
	struct sema* sem = (struct sema*) handle;
	__semaphore_lock(sem->lso, sem->lock);
}


void lso_semunlock(lso_sem_t handle)
{
	struct sema* sem = (struct sema*) handle;
	__semaphore_unlock(sem->lso, sem->lock);
}


void lso_release_sem(lso_sem_t *handle)
{
	struct sema* sem = (struct sema*) *handle;
	sema_free(*(sem->lock));
	free(sem);
	*handle = NULL;
}

