#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "litmus_so.h"
#include "litmus_srp_sem.h"
#include "so_srp_sem.h"

struct srp_sem {
	struct litmus_so* 	lso;
	srp_sema_id*		lock; /* ID of appropriate srp_semaphore in litmus kernel */
};

lso_error_t lso_create_srp_sem(lso_t handle, const char* name)
{
	struct litmus_so* lso = (struct litmus_so*) handle;
	lso_global_t semlock;
	char buf[32];	

	snprintf(buf, 32, "srp-sem/%s", name);
	
	/* check if it exists already */
	if (lso_get(handle, buf))
		return LSO_EXISTS;
	semlock = alloc_srp_semaphore(lso);
	if (lso_put_exclusive(handle, buf, semlock) != LSO_OK) {
		free_srp_semaphore(lso, semlock);
		return LSO_EXISTS;
	} else
		return LSO_OK;
}


lso_error_t lso_get_srp_sem(lso_t handle, const char* name, lso_srp_sem_t* sem_handle)
{
	struct srp_sem* sem;
	lso_global_t semlock;
	char buf[32];	

	snprintf(buf, 32, "srp-sem/%s", name);
	semlock = lso_get(handle, buf);

	if (!semlock)
		return LSO_NOT_FOUND;
	sem = (struct srp_sem*) malloc(sizeof(struct srp_sem));
	if (!sem)
		return LSO_NO_MEM;
	sem->lso  = (struct litmus_so*) handle;
	sem->lock = lso_local_ptr(handle, semlock);
	*sem_handle = sem;
	return LSO_OK;
}

void lso_srp_semlock(lso_srp_sem_t handle)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	__srp_semaphore_lock(sem->lso, sem->lock);
}


void lso_srp_semunlock(lso_srp_sem_t handle)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	__srp_semaphore_unlock(sem->lso, sem->lock);
}

void lso_reg_with_srp_sem(lso_srp_sem_t handle)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	__srp_semaphore_reg(sem->lso, sem->lock);
}

void lso_reg_other_with_srp_sem(lso_srp_sem_t handle, pid_t task_pid)
{
	struct srp_sem* sem = (struct srp_sem*) handle;
	__srp_semaphore_reg_other(sem->lso, sem->lock, task_pid);
}

void lso_release_srp_sem(lso_srp_sem_t *handle)
{
	struct srp_sem* sem = (struct srp_sem*) *handle;
	srp_sema_free(*(sem->lock));
	free(sem);
	*handle = NULL;
}

